#!/usr/bin/env python
# coding=utf8

# Rename mp3 files and tags from unicode to greeklish, etc
# O Neos Kitharodos <kitharodos@kithara.gr>

# Started: 04.06.07
# First working version: 13.06.07
# Really working 14.06.07 after fixing mutagen incompatibilities with iTunes (see below)
# First GUI (sort of...) version: 18.06.07
# First released version 0.9: 21.06.07

# Uses mutagen mp3 library from: http://www.sacredchao.net/quodlibet/wiki/Development/Mutagen

# Changed the following in id3.py to make compatible with Winamp and iTunes:
# comment-out        if usize > 2048:
# comment-out            framedata = BitPaddedInt.to_str(usize) + framedata.encode('zlib')
# comment-out            flags |= Frame.FLAG24_COMPRESS | Frame.FLAG24_DATALEN
# comment-out        datasize = BitPaddedInt.to_str(len(framedata), width=4)
# comment-out        header = pack('>4s4sH', type(frame).__name__, datasize, flags)
# insert:       header = pack('>4sLH', type(frame).__name__, usize, flags)


####################################################################

# Settings

renameMode = 0			# 0: no change, 1: cp1253 to unicode, 2: greek to greeklish, 3: greeklish to greek
renameFiles = 2			# 0: don't rename, 1: rename filenames themselves, 2: rename from mp3 tags
renameTags = 0			# 0: don't rename, 1: rename according to renameMode, 2: rename from filename

# if renameTags = 2, then the following defines how we can use the dir name:
dirMode = 1				# 0: ignore, 1: dirName = album - artist, 2: dirName = CD number
						# 3: dirName = album name, 4: dirName = genre

# Examples:
# renameMode = 1:		Ìßêçò ÈåïäùñÜêçò  -> Μίκης Θεοδωράκης
# renameMode = 2:		Μίκης Θεοδωράκης  -> Mikhs Theodorakhs
# renameMode = 3:		Mikhs Theodwrakhs -> Μικης Θεοδωρακης

####################################################################

import sys, os, re, fnmatch, mutagen.id3
from mutagen.id3 import *
from ctypes import *


u2grng = {			# greek to greeklish table
	u'Α' : 'A',
	u'Β' : 'B',
	u'Γ' : 'G',
	u'Δ' : 'D',
	u'Ε' : 'E',
	u'Ζ' : 'Z',
	u'Η' : 'H',
	u'Θ' : '8',
	u'Ι' : 'I',
	u'Κ' : 'K',
	u'Λ' : 'L',
	u'Μ' : 'M',
	u'Ν' : 'N',
	u'Ξ' : 'Ks',
	u'Ο' : 'O',
	u'Π' : 'P',
	u'Ρ' : 'R',
	u'Σ' : 'S',
	u'Τ' : 'T',
	u'Υ' : 'Y',
	u'Φ' : 'F',
	u'Χ' : 'X',
	u'Ψ' : 'Ps',
	u'Ω' : 'W',
	u'Ά' : 'A',
	u'Έ' : 'E',
	u'Ή' : 'H',
	u'Ί' : 'I',
	u'Ό' : 'O',
	u'Ύ' : 'Z',
	u'Ώ' : 'O',
	u'Ϊ' : 'I',
	u'Ϋ' : 'Y',
	u'α' : 'a',
	u'β' : 'b',
	u'γ' : 'g',
	u'δ' : 'd',
	u'ε' : 'e',
	u'ζ' : 'z',
	u'η' : 'h',
	u'θ' : '8',
	u'ι' : 'i',
	u'κ' : 'k',
	u'λ' : 'l',
	u'μ' : 'm',
	u'ν' : 'n',
	u'ξ' : 'Ks',
	u'ο' : 'o',
	u'π' : 'p',
	u'ρ' : 'r',
	u'σ' : 's',
	u'ς' : 's',
	u'τ' : 't',
	u'υ' : 'y',
	u'φ' : 'f',
	u'χ' : 'x',
	u'ψ' : 'ps',
	u'ω' : 'w',
	u'ά' : 'a',
	u'έ' : 'e',
	u'ή' : 'h',
	u'ί' : 'i',
	u'ό' : 'o',
	u'ύ' : 'y',
	u'ώ' : 'o',
	u'ϊ' : 'i',
	u'ϋ' : 'y',
	u'ΐ' : 'i',
	u'ΰ' : 'y',
	}

grAmbi = {			# ambivalent chars (same in greek and english)
	u'A' : u'Α',
	u'B' : u'Β',
	u'E' : u'Ε',
	u'H' : u'Η',
	u'I' : u'Ι',
	u'K' : u'Κ',
	u'M' : u'Μ',
	u'N' : u'Ν',
	u'O' : u'Ο',
	u'P' : u'Ρ',
	u'T' : u'Τ',
	u'X' : u'Χ',
	u'Y' : u'Υ',
	u'Z' : u'Ζ',
	}

tags = [		# we care about these tags only
	'TCOP',		# copyright
	'TALB',		# album
	'TIT2',		# track title
	'TPE1',		# artist
	'TOPE',		# original artist
	'TCON',		# genre
	'TPE2',		# album-artist
	'TCOM',		# composer
	]

grng2U_1 = {			# greeklish to greek table - pass 1
	'Ks' : u'Ξ',
#	'Th' : u'Θ',
	'Ps' : u'Ψ',
	'ks' : u'ξ',
#	'th' : u'θ',
	'ps' : u'ψ',
	}

grng2U_2 = {			# greeklish to greek table - pass 2
	'A' : u'Α',
	'B' : u'Β',
	'G' : u'Γ',
	'D' : u'Δ',
	'E' : u'Ε',
	'Z' : u'Ζ',
	'H' : u'Η',
	'Q' : u'Θ',
	'I' : u'Ι',
	'K' : u'Κ',
	'L' : u'Λ',
	'M' : u'Μ',
	'N' : u'Ν',
	'J' : u'Ξ',
	'O' : u'Ο',
	'P' : u'Π',
	'R' : u'Ρ',
	'S' : u'Σ',
	'T' : u'Τ',
	'Y' : u'Υ',
	'U' : u'Υ',
	'V' : u'Β',
	'F' : u'Φ',
	'X' : u'Χ',
#	'C' : u'Ψ',
	'W' : u'Ω',
	'a' : u'α',
	'b' : u'β',
	'c' : u'γ',
	'g' : u'γ',
	'd' : u'δ',
	'e' : u'ε',
	'z' : u'ζ',
	'h' : u'η',
	'i' : u'ι',
	'k' : u'κ',
	'l' : u'λ',
	'm' : u'μ',
	'n' : u'ν',
	'x' : u'ξ',
	'o' : u'ο',
	'p' : u'π',
	'r' : u'ρ',
	's' : u'σ',
	't' : u'τ',
	'y' : u'υ',
	'u' : u'υ',
	'v' : u'β',
	'f' : u'φ',
	'x' : u'χ',
#	'c' : u'ψ',
	'q' : u'ω',
	'w' : u'ω',
	}


examples = {			# example strings to display in the dialog
	'0' : u'( - )',
	'1' : u'Ìßêçò ÈåïäùñÜêçò  -> Μίκης Θεοδωράκης',
	'2' : u'Μίκης Θεοδωράκης  -> Mikhs Theodorakhs',
	'3' : u'Mikhs Theodwrakhs -> Μικης Θεοδωρακης',
}



####################################################################
import ctypes

def getLongPathName(path):								# excellent hack to get the long (unicode) name from a short dos-type (8.3) name
	buf = ctypes.create_unicode_buffer(500)
	GetLongPathName = ctypes.windll.kernel32.GetLongPathNameW
	rv = GetLongPathName(path, buf, 500)
	if rv == 0 or rv > 500:
		return path
	else:
		return buf.value



####################################################################
def strGrng2U(str):

	str = re.sub( u'([a-zA-Z])8', u'\\1θ', str)			# an8os -> anθos
	str = re.sub( u'8([a-zA-Z])', u'θ\\1', str)			# an8os -> anθos
	str = re.sub( u'([a-zA-Z])3', u'\\1ξ', str)			# el3h -> elξh
	str = re.sub( u'3([a-zA-Z])', u'ξ\\1', str)			# el3h -> elξh

	for c in grng2U_1.keys():
		str = str.replace(c, grng2U_1[c])

	for c in grng2U_2.keys():
		str = str.replace(c, grng2U_2[c])

	str = fixStel(str)
	return str




####################################################################
def stru2grng(str):

	for c in u2grng.keys():
		str = str.replace(c, u2grng[c])

	return str



####################################################################
def strGrAmbi(str):

	for c in grAmbi.keys():
		str = str.replace(c, grAmbi[c])

	return str



####################################################################
def sanitizePath(str):

	str = str.replace("/", "-")
	str = str.replace("&", "+")
	str = str.replace(" - ", "+")
	return str



####################################################################
def cleanupFname(str):

	str = str.replace("__", " - ")
	str = str.replace("_", " ")
	str = re.sub( '\s+', ' ', str)			# collapse spaces
	str = trim(str)
	return str



####################################################################
def fixStel(str):

	str = re.sub( u'σ([ .,\-+_\(\)\!\*])', u'ς\\1', str)
	str = re.sub( u'σ\Z', u'ς', str)
	return str



####################################################################
def trim(str):
	m = re.match( '^\s*(.*)\s*$', str )			# trim spaces
	if m:
		return m.group(1)
	else:
		return ""



####################################################################
def capitWord(str):

	r = ""
	for w in str.split():
		r = r + w.capitalize() + " " 

	return r[:-1]



####################################################################
def myLcase(str, each=False):

	if each:
		s1 = capitWord(str)
	else:
		s1 = str.capitalize()

	return fixStel(s1)



####################################################################
def str1253toU(str):

	if not isinstance(str, unicode):
		return unicode(str, 'cp1253')
	u = "";
	for i in range(len(str)):
		o = ord(str[i])
		if o < 255:
			plainCh = chr(o)
			uniCh = unicode(plainCh, 'cp1253')
		else:
			uniCh = unichr(o)
		u = u + uniCh
	u = strGrAmbi(u)
	return u



####################################################################
def multiConvert(str):

	global renameMode

	if renameMode == 0:	
		return str
	if renameMode == 1:	
		return str1253toU(str)
	if renameMode == 2:
		return stru2grng(str)
	if renameMode == 3:
		return strGrng2U(str)



##############################################################
def parseFName (fname):

	num = tit = art = ""
	m = re.match( '^(\d{1,2}) - (.+) - (.+)$', fname)			# 05 - Artist - Track
	if m:
		num = m.group(1)
		art = m.group(2)
		tit = m.group(3)

	m = re.match( '^(\d{1,2})[ \-](.+) - (.+)$', fname)			# 05-Artist - Track
	if m:
		num = m.group(1)
		art = m.group(2)
		tit = m.group(3)

	m = re.match( '^(\d{1,2})[ \-](.+)-(.+)$', fname)			# 05-Artist-Track
	if m:
		num = m.group(1)
		art = m.group(2)
		tit = m.group(3)

	m = re.match( '^(.+) - (\d{1,2}) - (.+)$', fname)			# Artist - 05 - Track
	if m:
		num = m.group(2)
		art = m.group(1)
		tit = m.group(3)

	m = re.match( '^(.+) - (\d{1,2})[ \-](.+)$', fname)			# Artist - 05-Track
	if m:
		num = m.group(2)
		art = m.group(1)
		tit = m.group(3)

	m = re.match( '^(.+)[ -](\d{1,2})[ \-](.+)$', fname)		# Artist-05-Track
	if m:
		num = m.group(2)
		art = m.group(1)
		tit = m.group(3)

	m = re.match( '^(\d{1,2}) - (.+)$', fname)					# 05 - Track
	if m:
		num = m.group(1)
		tit = m.group(2)

	m = re.match( '^(\d{1,2})[ \-](.+)$', fname)				# 05-Track
	if m:
		num = m.group(1)
		tit = m.group(2)

	m = re.match( '^(.+) - (.+)$', fname)						# Artist - Track
	if m:
		art = m.group(1)
		tit = m.group(2)

	m = re.match( '^(.+)-(.+)$', fname)							# Artist-Track
	if m:
		art = m.group(1)
		tit = m.group(2)

	if tit == "":
		print "++++ Μη-αναγνωρίσιμο όνομα:", fname.encode('utf8')
		tit = fname;

	num = trim(num)
	art = myLcase(trim(art), True)
	tit = myLcase(trim(tit))
	print "Αριθμός:", num.encode('utf8'), "καλλιτέχνης:", art.encode('utf8'), "τίτλος:", tit.encode('utf8')
	return (num, art, tit)



####################################################################


def myMain():

	global renameMode, renameFiles, renameTags, dirMode, dir

	mp3enc = 1							# unicode
	if renameMode == 2: mp2enc = 0		# ascii
	
	print '<head><meta http-equiv="Content-Type" content="text/html; charset=utf-8">'
	print '<title>fixMp3Tags: ' + dir.encode('utf8') + '</title>'
	print "</head>";
	print '<style type="text/css">pre { font-size: 12px; }</style>'
	print "<pre>";
	
	print "Επεξεργασία φάκελου:", dir.encode('utf8')
	
	for root, dirs, files in os.walk(dir, topdown=False):
		files.extend(dirs)
		for name in files:
			pathOld = os.path.join(root, name)
			pathNew = pathOld
			print "\nΕπεξεργασία αρχείου/φάκελου", pathNew.encode("utf8") + ":\n"
			(x, dirName) = os.path.split(root)
			(base, ext) = os.path.splitext(name)
			dirName = multiConvert(dirName)
			baseNew = multiConvert(base)
			if renameFiles == 1:
				pathNew = os.path.join(root, myLcase(baseNew, True) + ext)
				print "Μετονομασία σε:", pathNew.encode("utf8")
				os.rename(pathOld, pathNew)
			if fnmatch.fnmatch(pathNew, "*.mp3"):
							
				try: id3 = ID3(pathNew)
				except mutagen.id3.ID3NoHeaderError:
					print "Δεν βρέθηκαν ID3 tags, θα δημιουργηθούν νέα."
					id3 = mutagen.id3.ID3()
				dirty = 0
	
				if renameTags == 1:
					for tg in tags:
						if tg in id3:
							tgOld = id3[tg].text[0]
							if tg in ['TPE1', 'TOPE', 'TPE2', 'TCOM']:		# Capitalize all words
								tgNew = multiConvert(tgOld)
								tgNew = myLcase(tgNew, True)
							else:											# Capitalize only first
								tgNew = multiConvert(tgOld)
								tgNew = myLcase(tgNew)
								
							id3.delall(tg)
							frame = mutagen.id3.Frames[tg](encoding=mp3enc, text=tgNew)
							id3.add(frame)
							dirty = 1
							print tg, "ήταν", tgOld.encode('utf8'), ", θα μετατραπεί σε", tgNew.encode('utf8')
	
				if renameTags == 2 and renameFiles != 2:
					
					(num, art, tit) = parseFName( cleanupFname(baseNew) )
					frame = mutagen.id3.Frames['TPE1'](encoding=mp3enc, text=art)
					id3.add(frame)
					frame = mutagen.id3.Frames['TIT2'](encoding=mp3enc, text=tit)
					id3.add(frame)
					frame = mutagen.id3.Frames['TRCK'](encoding=mp3enc, text=num)
					id3.add(frame)
					
					if dirMode == 1:
						m = re.match('^(.+)\s*-\s*(.+)$', dirName)							# if dirName = album - artist
						art = m.group(1)
						alb = m.group(2)
						frame = mutagen.id3.Frames['TALB'](encoding=mp3enc, text=alb)
						id3.add(frame)
						id3.delall('TPE1')
						frame = mutagen.id3.Frames['TPE1'](encoding=mp3enc, text=art)
						id3.add(frame)
					if dirMode == 2:
						cdnum = re.match('(\d+)', dirName).group(1)							# if dirName = CD number
						frame = mutagen.id3.Frames['TPOS'](encoding=mp3enc, text=cdnum)
						id3.add(frame)
					if dirMode == 3:
						frame = mutagen.id3.Frames['TALB'](encoding=mp3enc, text=dirName)		# if dirName = album name
						id3.add(frame)
					if dirMode == 4:
						frame = mutagen.id3.Frames['TCON'](encoding=mp3enc, text=dirName)		# if dirName = genre
						id3.add(frame)
	
					dirty = 1				
					
				if dirty:
					print "Αποθήκευση tags..."
					id3.save(pathNew, v1=0)						# save and delete v1 tags
	
				if renameFiles == 2 and renameTags != 2:
					try: trck = id3['TRCK'].text[0]
					except KeyError: trck = ""
					trck = re.match('^(\d+)', trck).group(1)
					if len(trck) < 2: trck = "0" + trck
					try: arti = id3['TPE1'].text[0]
					except KeyError: arti = ""
					arti = multiConvert(arti)
					arti = sanitizePath(arti)
					try: titl = id3['TIT2'].text[0]
					except KeyError: titl = ""
					titl = multiConvert(titl)
					titl = sanitizePath(titl)
					newN = ""
					if trck != "": newN = trck + " "
					if arti != "": newN = newN + arti + " - "
					if titl != "": newN = newN + titl
					if len(newN) > 0:
						pathNew2 = os.path.join(root, newN + ".mp3")
						print "Μετονομασία αρχείου σε:", pathNew2.encode('utf8')
						os.rename(pathOld, pathNew2)
					
				print "Λίστα όλων των mp3 tags του αρχείου:"
	#			for frame in id3.values(): print repr(frame)		# dump all existing tags
				print id3.pprint().encode('utf8')					# pretty-pring all existing tags
	
	print "</pre>";
	exit()


####################################################################


from Tkinter import *

class App:

	def __init__(self, master):
		
		global m, f, t, d
		global dir
		
		master.title("Fix mp3 Gr - Μετατροπή ονομάτων και mp3 tags")
		master.config(bd=10)
		font = ("Tahoma", 8)
		
		self.fileLab = Label(master, text=u'Φάκελος/αρχείο:\n'+dir, fg='blue', font=font)
		self.fileLab.grid(row=0, column=1, columnspan=2, ipady=10, sticky=S)

		self.charLab = Label(master, text="1) Μετατροπή χαρακτήρων:", fg='blue', font=font)
		self.charLab.grid(row=1, column=1, ipady=10, sticky=W)

		MODES = [
			("Καμία αλλαγή", "0"),
			("cp1253 σε ελληνικά", "1"),
			("Ελληνικά σε greeklish", "2"),
			("Greeklish σε ελληνικά", "3"),
			]
		m = StringVar()
		m.set("0")
		for text, mode in MODES:
			self.mb = Radiobutton(master, text=text, variable=m, value=mode, font=font, borderwidth=0, command=self.setExample)
			self.mb.grid(column=1, sticky=W)
			
		self.fileLab = Label(master, text="2) Ονόματα αρχείων:", fg='blue', font=font)
		self.fileLab.grid(column=1, sticky=W)

		MODES = [
			("Καμία αλλαγή", "0"),
			("Μεταγραμματισμός σύμφωνα με το 1)", "1"),
			("Ονόματα αρχείων από mp3 tags", "2"),
			]
		f = StringVar()
		f.set("0")
		for text, mode in MODES:
			self.fb = Radiobutton(master, text=text, variable=f, value=mode, font=font, borderwidth=0, command=self.setTagButtState)
			self.fb.grid(column=1, sticky=W)

		self.tagLab = Label(master, text="3) Mp3 tags:", fg='blue', font=font)
		self.tagLab.grid(row=6, column=2, ipady=10, sticky=W)

		MODES = [
			("Καμία αλλαγή", "0"),
			("Μεταγραμματισμός σύμφωνα με το 1)", "1"),
			("Mp3 tags από ονόματα αρχείων", "2"),
			]
		t = StringVar()
		t.set("0")
		row = 7
		for text, mode in MODES:
			self.tb = Radiobutton(master, text=text, variable=t, value=mode, font=font, borderwidth=0, command=self.setDirButtState)
	 		self.tb.grid(row=row, column=2, sticky=W)
	 		row = row + 1

		self.dirLab = Label(master, text="4) Σημασία ονομάτων φακέλων:", fg='blue', font=font)
		self.dirLab.grid(row=12, column=2, ipady=10, sticky=W)

		MODES = [
			("Καμία αλλαγή", "0"),
			("Φάκελος = Άλμπουμ - Καλλιτέχνης", "1"),
			("Φάκελος = Αριθμός CD", "2"),
			("Φάκελος = Άλμπουμ", "3"),
			("Φάκελος = Είδος μουσικής", "4"),
			]
		d = StringVar()
		d.set("0")
		row = 13
		self.dbList = []
		for text, mode in MODES:
			self.db = Radiobutton(master, text=text, variable=d, value=mode, font=font, borderwidth=0, state=DISABLED)
	 		self.db.grid(row=row, column=2, sticky=W)
	 		row = row + 1
	 		self.dbList.append(self.db)					# keep those in list for activating / deactivating later

		self.cancel = Button(master, text="Άκυρο", command=self.quit, font=font)
		self.cancel.grid(row=19, column=2)
		
		self.ok = Button(master, text="Μετατροπή", command=self.do_it, font=font)
		self.ok.grid(row=19, column=1, pady=10)
				
		self.abLab = Label(master, text="fixMp3gr v.0.9 - Ο Νέος Κιθαρωδός <kitharodos@kithara.gr>", fg='gray', font=font)
		self.abLab.grid(row=20, column=1, columnspan=2)
				
		self.exLabel = Label(master, text="Παράδειγμα:", font=font)
		self.exLabel.grid(row=2, column=2, sticky=W)
		self.example = Label(master, text="( - )", font=font)
		self.example.grid(row=3, column=2, rowspan=2, sticky=NW)


	def do_it(self):
		global m, f, t, d
		global renameMode, renameFiles, renameTags, dirMode

		renameMode = int(m.get())
		renameFiles = int(f.get())
		renameTags = int(t.get())
		dirMode = int(d.get())
		myMain()


	def setExample(self):
		global m
		self.example.config(text=examples[m.get()])


	def setTagButtState(self):
		global m, f, t, d
		
		if f.get() == "2":
			self.tb.config(state=DISABLED)
		else:
			self.tb.config(state=NORMAL)


	def setDirButtState(self):
		global m, f, t, d
		
		if t.get() == "2":
			for rad in self.dbList:
				rad.config(state=NORMAL)
		else:
			for rad in self.dbList:
				rad.config(state=DISABLED)
		if t.get() == "2":
			self.fb.config(state=DISABLED)
		else:
			self.fb.config(state=NORMAL)


	def quit(self):
		exit()



####################################################################


HANDLE = c_ulong
HWND = HANDLE

BIF_RETURNONLYFSDIRS = 0x0001   # For finding a folder to start document
                                # searching
BIF_DONTGOBELOWDOMAIN = 0x0002  # For starting the Find Computer
BIF_STATUSTEXT = 0x0004         # Top of the dialog has 2 lines of text for
                                # BROWSEINFO.lpszTitle and one line if this flag
                                # is set.  Passing the message
                                # BFFM_SETSTATUSTEXTA to the hwnd can set the
                                # rest of the text.  This is not used with
                                # BIF_USENEWUI and BROWSEINFO.lpszTitle gets
                                # all three lines of text.
BIF_RETURNFSANCESTORS = 0x0008
BIF_EDITBOX = 0x0010            # Add an editbox to the dialog
BIF_VALIDATE = 0x0020           # insist on valid result (or CANCEL)
BIF_NEWDIALOGSTYLE = 0x0040     # Use the new dialog layout with the ability to
                                # resize. # Caller needs to call OleInitialize()
                                # before using this API
BIF_USENEWUI = (BIF_NEWDIALOGSTYLE | BIF_EDITBOX)
BIF_BROWSEINCLUDEURLS = 0x0080  # Allow URLs to be displayed or entered.
                                # (Requires BIF_USENEWUI)
BIF_UAHINT = 0x0100             # Add a UA hint to the dialog, in place of the
                                # edit box. May not be combined with BIF_EDITBOX
BIF_NONEWFOLDERBUTTON = 0x0200  # Do not add the "New Folder" button to the
                                # dialog. Only applicable with
                                # BIF_NEWDIALOGSTYLE.
BIF_NOTRANSLATETARGETS = 0x0400 # don't traverse target as shortcut
BIF_BROWSEFORCOMPUTER = 0x1000  # Browsing for Computers.
BIF_BROWSEFORPRINTER = 0x2000   # Browsing for Printers
BIF_BROWSEINCLUDEFILES = 0x4000 # Browsing for Everything
BIF_SHAREABLE = 0x8000          # sharable resources displayed (remote shares,
                                # requires BIF_USENEWUI)

class _ITEMIDLIST(Structure):
	pass

ITEMIDLIST = _ITEMIDLIST
LPCITEMIDLIST = POINTER(ITEMIDLIST)

class BROWSEINFO(Structure):
	_fields_ = [
	('hwndOwner', HWND),
	('pidlRoot', LPCITEMIDLIST),
	('pszDisplayName', c_wchar_p),
	('lpszTitle', c_wchar_p),
	('ulFlags', c_uint),
	('lpfn', c_int),
	('lParam', c_long),
	('iImage', c_int)]

def SHBrowseForFolder(title, handle=0, new_folder_button=False):
	flags = 0
	bi = BROWSEINFO()
	flags = BIF_RETURNONLYFSDIRS | BIF_NEWDIALOGSTYLE | BIF_UAHINT
	if not new_folder_button:
		flags = flags | BIF_NONEWFOLDERBUTTON
	bi.ulFlags = flags
	bi.hwndOwner = handle
	bi.pidlRoot = None
	bi.lpszTitle = title

	windll.ole32.OleInitialize(None)

	pidl = windll.shell32.SHBrowseForFolderW(byref(bi))

	buffer = ctypes.create_unicode_buffer(255)
	# ret = 1 if folder selected, 0 otherwise.
	ret = windll.shell32.SHGetPathFromIDListW(pidl, byref(buffer))
	if ret:
		selected_folder = buffer.value
	else:
		selected_folder = None
	windll.ole32.CoTaskMemFree(pidl)
	windll.ole32.OleUninitialize()
	return selected_folder




####################################################################
# Main program
####################################################################

try:
	dir = unicode(sys.argv[1])		# if called from the command line (batch file)
except IndexError:
	dir = ""
else:
	dir = os.path.abspath(dir)
	dir = getLongPathName(dir)

root = Tk()

if len(dir) == 0:
	title = u"Επιλέξτε ένα φάκελο για τη μετατροπή:"
	dir = SHBrowseForFolder(title)
	if dir is None:
		exit()

app = App(root)

root.mainloop()

exit()


####################################################################
